/*---------- LISTING 2 ------------------------ ELLIPSE.C -----------*
* Desc:       16-Bit Ellipse Drawing Algorithm                       *
* Author:     Marv Luse, Ithaca Street Software, Inc.                *
* Date:       January, 1990                                          *
* Copyright:  Use freely, but acknowledge authorship                 *
* Notes:      SET_PIXEL, MOVE_TO, and LINE_TO should be defined      *
*             appropriately for compiler and graphics calls used.    *
* Compilers:  Turbo C 2.0 and Microsoft C 5.1                        *
* Switches:   #define these as needed                                *
*             BGI - for Borland's Graphics Interface                 *
*             MSC - for Microsoft's graphics routines                *
*             META - for MetaWINDOW's display routines               *
*                                                                    *
*             TEST - to get a test driver (Turbo C only)             *
*--------------------------------------------------------------------*/

#include <stdlib.h>
#include <stdio.h>
#include <conio.h>

/* Borland BGI Graphics */
#ifdef BGI
#include <graphics.h>
#define SET_PIXEL( x, y )  putpixel( x, y, 15 )
#define MOVE_TO( x, y )    moveto( x, y )
#define LINE_TO( x, y )    lineto( x, y )
#endif

/* Microsoft C Graphis */
#ifdef MSC
#include <graph.h>
#define SET_PIXEL( x, y )  _setpixel( x, y )
#define MOVE_TO( x, y )    _moveto( x, y )
#define LINE_TO( x, y )    _lineto( x, y )
#endif

/* Metagraphics MetaWINDOW */
#ifdef META
#define SET_PIXEL( x, y )  SetPixel( x, y )
#define MOVE_TO( x, y )    MoveTo( x, y )
#define LINE_TO( x, y )    LineTo( x, y )
#endif

/*--------------------------------------------------------------------*/
/* Draw a vertical line segment                                       */

void SetYPx( int x, int y1, int y2 )
{
    MOVE_TO( x, y1 );  LINE_TO( x, y2 );
}

/*--------------------------------------------------------------------*/
/* Draw a horizontal line segment                                     */

void SetXPx( int x1, int x2, int y )
{
    MOVE_TO( x1, y );  LINE_TO( x2, y );
}

/*--------------------------------------------------------------------*/
/* Horizontally aligned ellipse : s wide by r high, r <= s            */

void HEllipse( int xc, int yc, int s, int r )
{
    int err, Dx, Dy, xds, yds, xdr, ydr, xslast, yslast;

    xds = 1;  xdr = 1;  xslast = 1;
    yds = s;  ydr = r;  yslast = s;
    err = ( -(r+s) ) >> 1;
    Dx = r - s;
    Dy = r - s;

    SET_PIXEL( xc-s, yc );
    SET_PIXEL( xc+s, yc );
    SET_PIXEL( xc, yc-r );
    SET_PIXEL( xc, yc+r );

    while( xdr < ydr ) {
        if ( err > 0 ) {
            err += ((xds-yds)<<2) + 10;
            yds--;
            Dy += r;
            if( Dy > 0 ) {
                SetXPx(xc+xds, xc+xslast, yc+ydr);
                SetXPx(xc-xds, xc-xslast, yc-ydr);
                SetXPx(xc+xds, xc+xslast, yc-ydr);
                SetXPx(xc-xds, xc-xslast, yc+ydr);
                xslast = xds;
                ydr--;
                Dy -= s;
            }
        }
        else
            err += (xds<<2) + 6;

        xds++;
        Dx += r;
        if( Dx > 0 ) {
            SetXPx( xc+yds, xc+yslast, yc+xdr );
            SetXPx( xc-yds, xc-yslast, yc-xdr );
            SetXPx( xc+yds, xc+yslast, yc-xdr );
            SetXPx( xc-yds, xc-yslast, yc+xdr );
            yslast = yds;
            xdr++;
            Dx-=s;
        }
    }

    if( xslast != yslast ) {
        SetXPx(xc+yslast, xc+xslast, yc+ydr);
        SetXPx(xc-yslast, xc-xslast, yc-ydr);
        SetXPx(xc+yslast, xc+xslast, yc-ydr);
        SetXPx(xc-yslast, xc-xslast, yc+ydr);
    }
}

/*--------------------------------------------------------------------*/
/* Horizontally aligned ellipse : s wide by r high, r > s             */

void VEllipse( int xc, int yc, int s, int r )
{
    int err, Dx, Dy, xds, yds, xdr, ydr, xrlast, yrlast;

    ydr = 1;  yds = 1;  yrlast = 1;
    xdr = r;  xds = s;  xrlast = r;
    err = ( -(r+s) ) >> 1;
    Dx = s - r;
    Dy = s - r;

    SET_PIXEL( xc-s, yc );
    SET_PIXEL( xc+s, yc );
    SET_PIXEL( xc, yc-r );
    SET_PIXEL( xc, yc+r );

    while( yds < xds ) {
        if ( err > 0 ) {
            err += ((ydr-xdr)<<2) + 10;
            xdr--;
            Dx += s;
            if( Dx > 0 ) {
                SetYPx( xc-xds, yc+ydr, yc+yrlast );
                SetYPx( xc+xds, yc+ydr, yc+yrlast );
                SetYPx( xc-xds, yc-ydr, yc-yrlast );
                SetYPx( xc+xds, yc-ydr, yc-yrlast );
                yrlast = ydr;
                xds--;
                Dx -= r;
            }
        }
        else
            err += (ydr<<2) + 6;

        ydr++;
        Dy += s;
        if( Dy > 0 ) {
            SetYPx( xc+yds, yc+xdr, yc+xrlast );
            SetYPx( xc-yds, yc+xdr, yc+xrlast );
            SetYPx( xc+yds, yc-xdr, yc-xrlast );
            SetYPx( xc-yds, yc-xdr, yc-xrlast );
            xrlast = xdr;
            yds++;
            Dy -= r;
        }
    }

    if( xrlast != yrlast ) {
        SetYPx( xc+xds, yc+xrlast, yc+yrlast );
        SetYPx( xc-xds, yc+xrlast, yc+yrlast );
        SetYPx( xc+xds, yc-xrlast, yc-yrlast );
        SetYPx( xc-xds, yc-xrlast, yc-yrlast );
    }
}

/*--------------------------------------------------------------------*/
/* Ellipse - generic entry point                                      */

void Ellipse( int xc, int yc, int s, int r )
{
    if( r > s )
        VEllipse( xc, yc, s, r );
    else
        HEllipse( xc, yc, s, r );
}

#pragma page()
#ifdef TEST

#ifndef BGI
#error Test driver is only available for Turbo C
#endif

/*--------------------------------------------------------------------*/
/* Function to initialize the Turbo C graphics environment            */

char *GrafPath = "E:\\TURBOC\\";     /* Change as needed */
int   GrafDriver, GrafMode, GrafError;

void GrafInit ( void )
{
    GrafDriver = DETECT;
    GrafMode = 0;
    initgraph(&GrafDriver, &GrafMode, GrafPath);
    GrafError = graphresult();
    if ( GrafError != grOk ) {
        printf ("\nGraphics System Error: %s", grapherrormsg(GrafError));
        printf ("\nExecution Terminated");
        exit ( 8 );
    }
}

/*--------------------------------------------------------------------*/
/* Function to terminate the graphics environment                     */

void GrafTerm ( void )
{
    closegraph();
}

/*--------------------------------------------------------------------*/
/* Program to illustrate the ellipse drawing code                     */

void main ( void )
{
    int x, y, r, s;

    GrafInit();
    x = getmaxx() / 2;
    y = getmaxy() / 2;

    r = y - 50;
    s = r + 50;
    Ellipse( x, y, s, r );
    Ellipse( x, y, s/2, r/2 );
    getch();

    r = y - 50;
    s = r - 50;
    Ellipse( x, y, s, r );
    Ellipse( x, y, s/2, r/2 );
    getch();

    GrafTerm();
}
#endif